// -----------------------------------------------------------------------
// <copyright file="IComparerTests.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Spaanjaars.ContactManager.BusinessEntities;

namespace Spaanjaars.Validation.Tests
{
	#region Helpers

	public class OrderLineCollection : BusinessCollectionBase<OrderLine>
	{
	}

	public class OrderLine : ValidationBase
	{
		public double ItemPrice { get; set; }
		public int Quantity { get; set; }

		public class OrderLineComparer : IComparer<OrderLine>
		{
			private string _sortColumn;
			private bool _reverse;
			public OrderLineComparer(string sortExpression)
			{
				if (string.IsNullOrEmpty(sortExpression))
				{
					sortExpression = "ItemPrice";
				}
				_reverse = sortExpression.ToLowerInvariant().EndsWith(" desc");
				if (_reverse)
				{
					_sortColumn = sortExpression.Substring(0, sortExpression.Length - 5);
				}
				else
				{
					_sortColumn = sortExpression;
				}
			}

			public int Compare(OrderLine lhs, OrderLine rhs)
			{
				int retVal = 0;
				switch (_sortColumn.ToLower())
				{
					case "itemprice":
						retVal = lhs.ItemPrice.CompareTo(rhs.ItemPrice);
						break;
					case "quantity":
						retVal = lhs.Quantity.CompareTo(rhs.Quantity);
						break;
					case "total":
						retVal = (lhs.ItemPrice * lhs.Quantity).CompareTo(rhs.ItemPrice * rhs.Quantity);
						break;
					default:
						throw new ArgumentException(string.Format("Unkown SortColumn: {0}", _sortColumn));
				}

				int _reverseInt = 1;
				if ((_reverse))
				{
					_reverseInt = -1;
				}
				return (retVal * _reverseInt);
			}
		}
	}
	#endregion

	/// <summary>
	/// Tests for the IComparer class of an OrderLine instance.
	/// </summary>
	[TestClass]
	public class IComparerTests
	{
		private OrderLineCollection GetOrderLines()
		{
			OrderLineCollection myList = new OrderLineCollection();
			myList.Add(new OrderLine() { ItemPrice = 10, Quantity = 4 });
			myList.Add(new OrderLine() { ItemPrice = 2, Quantity = 10 });
			myList.Add(new OrderLine() { ItemPrice = 6, Quantity = 5 });
			return myList;
		}

		[TestMethod]
		public void OrderOnQuantityTest()
		{
			OrderLineCollection myList = GetOrderLines();
			// standard order is 4, 10, 5. Let's assert that first
			Assert.AreEqual<int>(4, myList[0].Quantity);
			Assert.AreEqual<int>(10, myList[1].Quantity);
			Assert.AreEqual<int>(5, myList[2].Quantity);

			// Now sort on Quantity and check if the collection is sorted correctly.
			myList.Sort(new OrderLine.OrderLineComparer("Quantity"));
			Assert.AreEqual<int>(4, myList[0].Quantity);
			Assert.AreEqual<int>(5, myList[1].Quantity);
			Assert.AreEqual<int>(10, myList[2].Quantity);

			// Now sort on Quantity in reverse and check if the collection is sorted correctly.
			myList.Sort(new OrderLine.OrderLineComparer("Quantity desc"));
			Assert.AreEqual<int>(10, myList[0].Quantity);
			Assert.AreEqual<int>(5, myList[1].Quantity);
			Assert.AreEqual<int>(4, myList[2].Quantity);
		}

		[TestMethod]
		public void OrderOnItemPriceTest()
		{
			OrderLineCollection myList = GetOrderLines();
			// Standard order is 10, 2, 6. Let's assert that first.
			Assert.AreEqual<double>(10, myList[0].ItemPrice);
			Assert.AreEqual<double>(2, myList[1].ItemPrice);
			Assert.AreEqual<double>(6, myList[2].ItemPrice);

			// Now sort on ItemPrice and check if the collection is sorted correctly.
			myList.Sort(new OrderLine.OrderLineComparer("ItemPrice"));
			Assert.AreEqual<double>(2, myList[0].ItemPrice);
			Assert.AreEqual<double>(6, myList[1].ItemPrice);
			Assert.AreEqual<double>(10, myList[2].ItemPrice);

			// Now sort on ItemPrice in reverse and check if the collection is sorted correctly.
			myList.Sort(new OrderLine.OrderLineComparer("ItemPrice desc"));
			Assert.AreEqual<double>(10, myList[0].ItemPrice);
			Assert.AreEqual<double>(6, myList[1].ItemPrice);
			Assert.AreEqual<double>(2, myList[2].ItemPrice);
		}

		[TestMethod]
		[ExpectedException(typeof(InvalidOperationException))]
		public void CannotPassAscExplicitlyTest()
		{
			OrderLineCollection myList = GetOrderLines();
			// Now sort on ItemPrice and check if the collection is sorted correctly.
			myList.Sort(new OrderLine.OrderLineComparer("ItemPrice asc"));
		}

		[TestMethod]
		[ExpectedException(typeof(InvalidOperationException))]
		public void CannotPassUnknownColumnTest()
		{
			OrderLineCollection myList = GetOrderLines();
			// Now sort on ItemPrice and check if the collection is sorted correctly.
			myList.Sort(new OrderLine.OrderLineComparer("DoesNotExist"));
		}

		[TestMethod]
		public void PassingStringEmptySortsOnItemPriceTest()
		{
			OrderLineCollection myList = GetOrderLines();
			// Now sort on string.Empty and check if the collection is sorted correctly
			// based on the default of ItemPrice.
			myList.Sort(new OrderLine.OrderLineComparer(string.Empty));
			Assert.AreEqual<double>(2, myList[0].ItemPrice);
			Assert.AreEqual<double>(6, myList[1].ItemPrice);
			Assert.AreEqual<double>(10, myList[2].ItemPrice);
		}

		[TestMethod]
		public void SortingOnDerivedDataTest()
		{
			OrderLineCollection myList = GetOrderLines();
			// List is now in order: 40, 20, 30.
			// Let's sort on Total ascending so it ends up as 20, 30, 40
			myList.Sort(new OrderLine.OrderLineComparer("Total"));
			Assert.AreEqual<double>(20d, myList[0].ItemPrice * myList[0].Quantity);
			Assert.AreEqual<double>(30d, myList[1].ItemPrice * myList[1].Quantity);
			Assert.AreEqual<double>(40d, myList[2].ItemPrice * myList[2].Quantity);

			// Now let's sort on Total in descending order so it ends up as 40, 30, 20
			myList.Sort(new OrderLine.OrderLineComparer("Total desc"));
			Assert.AreEqual<double>(40d, myList[0].ItemPrice * myList[0].Quantity);
			Assert.AreEqual<double>(30d, myList[1].ItemPrice * myList[1].Quantity);
			Assert.AreEqual<double>(20d, myList[2].ItemPrice * myList[2].Quantity);
		}
	}
}
