// -----------------------------------------------------------------------
// <copyright file="PersonTests.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Spaanjaars.Validation.Tests
{
	#region Helpers

	/// <summary>
	/// Helper class for simple Validation tests.
	/// </summary>
	public class Person : ValidationBase
	{
		[NotNullOrEmpty(Message = "First name is required.")]
		public string FirstName { get; set; }

		[NotNullOrEmpty(Message = "Last name is required.")]
		public string LastName { get; set; }

		[NotNullOrEmpty(Message = "Email address is required.")]
		[ValidEmail(Message = "A valid e-mail address is required.")]
		public string EmailAddress { get; set; }

		[ValidRange(Message = "Age must be between 0 and 150.", Min = 0, Max = 150)]
		public int Age { get; set; }
	}

	#endregion

	/// <summary>
	/// This class contains Unit Tests to test ValidationBase classes and the 
	/// validation framework.
	/// </summary>
	[TestClass]
	public class PersonTests
	{
		[TestMethod]
		public void NewPersonIsNotValidTest()
		{
			Person myPerson = new Person();
			Assert.IsFalse(myPerson.Validate());
		}

		[TestMethod]
		public void NullIsNotAValidFirstNameTest()
		{
			Person myPerson = new Person { FirstName = null };
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("FirstName").Count > 0);
		}

		[TestMethod]
		public void EmptyIsNotAValidFirstNameTest()
		{
			Person myPerson = new Person { FirstName = string.Empty };
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("FirstName").Count > 0);
		}

		[TestMethod]
		public void ImarIsAValidFirstNameTest()
		{
			Person myPerson = new Person { FirstName = "Imar" };
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("FirstName").Count == 0);
		}

		[TestMethod]
		public void AgeCannotBeMinusOne()
		{
			Person myPerson = new Person { Age = -1 };
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Age").Count > 0);
		}

		[TestMethod]
		public void AgeCannotBeLargeNegativeTest()
		{
			Person myPerson = new Person { Age = int.MinValue };
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Age").Count > 0);
		}

		[TestMethod]
		public void AgeCannotMoreThan150Test()
		{
			Person myPerson = new Person { Age = 151 };
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Age").Count > 0);
		}

		[TestMethod]
		public void AgeCannotBeLargePositiveTest()
		{
			Person myPerson = new Person { Age = int.MaxValue };
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Age").Count > 0);
		}

		[TestMethod]
		public void AgeCanBeLowerboundTest()
		{
			Person myPerson = new Person { Age = 0 };
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Age").Count == 0);
		}

		[TestMethod]
		public void AgeCanBeUpperboundTest()
		{
			Person myPerson = new Person { Age = 150 };
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Age").Count == 0);
		}

		[TestMethod]
		public void AgeCanBeBetweenLowerAndUpperboundsTest()
		{
			Person myPerson = new Person { Age = 130 };
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Age").Count == 0);
		}
	}
}
