// -----------------------------------------------------------------------
// <copyright file="ProjectTests.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Spaanjaars.Validation;

namespace Spaanjaars.Validation.Tests
{
	#region Helpers

	/// <summary>
	/// Helper class for tests of custom validation rules.
	/// </summary>
	public class Project : ValidationBase
	{
		[NotNullOrEmpty(Message = "Enter a start date.")]
		public DateTime? StartDate { get; set; }
		[NotNullOrEmpty(Message = "Enter an end date.")]
		public DateTime? EndDate { get; set; }

		public override bool Validate()
		{
			BrokenRules.Clear();
			bool localValid = true;
			if (StartDate >= EndDate)
			{
				BrokenRules.Add(@"The StartDate of this project has to be smaller than the EndDate.");
				localValid = false;
			}
			return base.Validate(false) && localValid;
		}
	}
	#endregion

	/// <summary>
	/// This class contains Unit Tests to test  ValidationBase classes and the 
	/// validation framework with custom validation behavior.
	/// </summary>
	[TestClass]
	public class ProjectTests
	{
		[TestMethod]
		public void EmptyDateIsNotValidTest()
		{
			Project myProject = new Project();
			Assert.IsFalse(myProject.Validate());
			Assert.IsTrue(myProject.BrokenRules.FindByPropertyName("StartDate").Count > 0);
		}

		[TestMethod]
		public void TodayIsValidDateTest()
		{
			Project myProject = new Project() { StartDate = DateTime.Now };
			myProject.Validate();
			Assert.IsTrue(myProject.BrokenRules.FindByPropertyName("StartDate").Count == 0);
		}

		[TestMethod]
		public void ProjectCannotEndBeforeStartTest()
		{
			Project myProject = new Project() { StartDate = DateTime.Now, EndDate = DateTime.Now.AddYears(-1) };
			Assert.IsFalse(myProject.Validate());
			Assert.IsTrue(myProject.BrokenRules.FindByMessage("The StartDate of this project has to be smaller than the EndDate.").Count > 0);
		}

		[TestMethod]
		public void ProjectCannotStartAndEndAtTheSameDateTest()
		{
			DateTime now = DateTime.Now;
			Project myProject = new Project() { StartDate = now, EndDate = now };
			Assert.IsFalse(myProject.Validate());
			Assert.IsTrue(myProject.BrokenRules.FindByMessage("The StartDate of this project has to be smaller than the EndDate.").Count > 0);
		}

		[TestMethod]
		public void ValidProjectTest()
		{
			Project myProject = new Project() { StartDate = DateTime.Now, EndDate = DateTime.Now.AddYears(1) };
			myProject.Validate();
			Assert.IsTrue(myProject.BrokenRules.FindByMessage("The StartDate of this project has to be smaller than the EndDate.").Count == 0);
		}
	}
}
