// -----------------------------------------------------------------------
// <copyright file="ValidationAttributeTests.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Spaanjaars.Validation;

namespace Spaanjaars.Validation.Tests
{
	#region Helpers

	/// <summary>
	/// Helper class for tests of validation attributes.
	/// </summary>
	public class LocalPerson : ValidationBase
	{
		[NotNullOrEmpty(Message = "Enter a first name")]
		public string FirstName { get; set; }

		public string LastName { get; set; }

		[ValidRange(Message = @"This is not a valid value. Valid values are between 0 and 10", Min = 0, Max = 10)]
		public int Priority { get; set; }

		[NotNullOrEmpty(Message = "Provide an object")]
		public object NotNullableObject { get; set; }

	}

	#endregion

	/// <summary>
	/// This class contains Unit Tests to test validation attributes.
	/// </summary>
	[TestClass]
	public class ValidationAttributeTests
	{
		#region Validation Properties

		[TestMethod]
		[ExpectedException(typeof(ArgumentException))]
		public void CannotSetMessageAndKeyAtTheSameTimeTest()
		{
			NotNullOrEmptyAttribute myAttribute = new NotNullOrEmptyAttribute();
			myAttribute.Key = "Key";
			myAttribute.Message = "Message";
		}

		[TestMethod]
		[ExpectedException(typeof(ArgumentException))]
		public void CannotSetKeyAndMessageAtTheSameTimeTest()
		{
			NotNullOrEmptyAttribute myAttribute = new NotNullOrEmptyAttribute();
			myAttribute.Message = "Message";
			myAttribute.Key = "Key";
		}

		[TestMethod]
		public void CanSetKeyTest()
		{
			NotNullOrEmptyAttribute myAttribute = new NotNullOrEmptyAttribute();
			myAttribute.Key = "Key";
			Assert.AreEqual("Key", myAttribute.Key);
		}

		[TestMethod]
		public void CanSetMessageTest()
		{
			NotNullOrEmptyAttribute myAttribute = new NotNullOrEmptyAttribute();
			myAttribute.Message = "Message";
			Assert.AreEqual("Message", myAttribute.Message);
		}


		#endregion

		#region NotNullOrEmpty Attribute


		[TestMethod]
		public void SupplyingAnObjectSatifiesObjectAttributeTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.NotNullableObject = new object();
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("NotNullableObject").Count == 0);
		}

		[TestMethod]
		public void SupplyingNoObjectTest()
		{
			LocalPerson myPerson = new LocalPerson();
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("NotNullableObject").Count > 0);
		}


		[TestMethod]
		public void NullNotValidTest()
		{
			LocalPerson myPerson = new LocalPerson();
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("FirstName").Count > 0);
		}

		[TestMethod]
		public void EmptyNotValidTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.FirstName = string.Empty;
			Assert.IsFalse(myPerson.Validate());
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("FirstName").Count > 0);
		}

		[TestMethod]
		public void ValidValueIsValidTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.FirstName = "Imar";
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("FirstName").Count == 0);
		}

		[TestMethod]
		public void PropertiesWithoutAttributesAreValidTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("LastName").Count == 0);
		}
		#endregion

		#region ValidRange Attribute

		[TestMethod]
		public void ValidPriorityTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Priority").Count == 0);
		}

		[TestMethod]
		public void ValidPriorityEdgeCaseMinTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Priority = 0;
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Priority").Count == 0);
		}

		[TestMethod]
		public void ValidPriorityEdgeCaseMaxTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Priority = 10;
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Priority").Count == 0);
		}

		[TestMethod]
		public void InvalidPriorityEdgeCaseMinTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Priority = -1;
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Priority").Count > 0);
		}

		[TestMethod]
		public void InvalidPriorityEdgeCaseMaxTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Priority = 11;
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Priority").Count > 0);
		}

		[TestMethod]
		public void InvalidPriorityInvalidMinTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Priority = -23441;
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Priority").Count > 0);
		}

		[TestMethod]
		public void InvalidPriorityInvalidMaxTest()
		{
			LocalPerson myPerson = new LocalPerson();
			myPerson.Priority = 4541;
			myPerson.Validate();
			Assert.IsTrue(myPerson.BrokenRules.FindByPropertyName("Priority").Count > 0);
		}

		#endregion
	}
}
